import matplotlib.pyplot as plt

# parameters
alpha = 0.1 # intrinsic rate of prey population increase
beta = 0.01 # predation rate coefficient
gamma = 0.05 # predator mortality rate
delta = 0.001 # reproduction rate of predators per 1 prey eaten
rab0 = 50 # initial rabbit population
fox0 = 15 # initial predator population
timelimit = 400 # number of timesteps to graph

# Lotka-Volterra equations
def rabRate(rab, fox):
    return rab * (alpha - beta * fox)

def foxRate(rab, fox):
    return rab * (gamma - delta * fox)

# takes current populations and returns tuple of new populations
def pop(rab, fox):
    # second-order Runge-Kutta method
    # ensure population is never negative
    rabMid = max(rab + 0.5 * rabRate(rab, fox), 0)
    foxMid = max(fox + 0.5 * foxRate(rab, fox), 0)
    
    rabNew = max(rab + rabRate(rabMid, foxMid), 0)
    foxNew = max(rab + foxRate(rabMid, foxMid), 0)
    return (rabNew, foxNew)

# initialize time-population lists with populations at time 0
rabPoints = [rab0]
foxPoints = [fox0]

# calculate new populations and add the data onto the lists
for t in range(1, timelimit):
    rabCur = rabPoints[-1] # last list element
    foxCur = foxPoints[-1] # last list element
    # calculate new populations
    popTuple = pop(rabCur, foxCur)
    rabNext = popTuple[0]
    foxNext = popTuple[1]

    # lists are of population (y-axis)
    rabPoints.append(rabNext)
    foxPoints.append(foxNext)

# plot populations versus time, rabbits in orange, foxes in blue
plt.plot(rabPoints, color="orange")
plt.plot(foxPoints, color="blue")
plt.show()
